'''
This module used by maxwelld vedro plugin.

All tasks for maxwelld service container transported via http transport.

Every command on service has it own endpoint with param set, described in maxwelld/server
'''
